// ceptimus 2017-12-19
// demo of "1.54 Inch 200x200 E-Ink Display Module Partial Refresh E-paper" module from Banggood.com

// ***WARNING*** use 3.3V Arduino, or use a Logic Level Converter Module with 5V Arduinos
// see ePaper.h file for connection details

#include "ePaper.h"

// bad rows at 170, 174, 182

void demo1() { // text and "parabolas"
  ePaper::clearBuffer(true);
  // this is to draw the "smile" under the Banggood text
  ePaper::filledCircle(55, 11, 19, false); // draw a big black circle
  ePaper::filledCircle(55, 5, 23, true); // draw a larger white circle rubbing out most of the black one but leaving the "smile" crescent behind
  
  ePaper::text(3, 4, "Bang", false); // the text
  ePaper::text(57, 4, "good", false);
  ePaper::textSmall(74, 22, ".com", false);
  ePaper::text(116, 4, "E-", false);
  ePaper::text(134, 4, "paper", false);
  ePaper::text(12, 183, "ceptimus.co.uk", false);
  
  ePaper::rectangle(34, 39, 164, 169, false); // the outer frame for the animation
  ePaper::rectangle(35, 40, 163, 168, false);

  ePaper::wake();
  ePaper::fullUpdateMode();
  ePaper::displayFrame(); // display everything drawn so far

  ePaper::partialUpdateMode(); // switch to partial update mode so that the lines in the box appear one at a time

  int p1x, p1y, p2x, p2y; // coordinates of the end points of the lines
  for (int i = 0; i <= 512; i += 16) {
    int d = i % 128L; // offset of end of line from a corner
    if (!d) continue; // skip drawing lines that are horizontal or vertical
    if (i <= 128) {
      p1x = 35 + d;
      p1y = 40;
      p2x = 163;
      p2y = 40 + d;
    } else if (i <= 256) {
      p1x = 163;
      p1y = 40 + d;
      p2x = 163 - d;
      p2y = 168;
    } else if (i <= 384) {
      p1x = 163 - d;
      p1y = 168;
      p2x = 35;
      p2y = 168 - d;
    } else {
      p1x = 35;
      p1y = 168 - d;
      p2x = 35 + d;
      p2y = 40;
    }
    ePaper::line(p1x, p1y, p2x, p2y, false);
    ePaper::displayFrame();
  }
  ePaper::sleep();
}

void demo2() { // circles
  ePaper::clearBuffer(true);
  randomSeed(analogRead(0) + millis()); // if analog pin 0 is unconnected, this should produce different random circles each time
  ePaper::circle(random(100) + 50, random(50) + 50, random(50) + 20, false);
  ePaper::wake();
  ePaper::fullUpdateMode(); // do a full update the first time
  ePaper::displayFrame(); // display everything drawn so far
  ePaper::partialUpdateMode(); // partial update mode from now on for speed and to reduce flickering

  for (uint32_t stopMillis = millis() + 15000; millis() < stopMillis; ) { // loop for fifteen seconds
    int x = random(220) - 10;
    int y = random(220) - 10;
    int radius = random(95) + 5;
    if (radius < 30) { // just for variety, draw a "washer" rather than a circle when radius is less than 30
      ePaper::filledCircle(x, y, radius, false); // first draw a solid circle in black
      ePaper::filledCircle(x, y, radius / 2, true); // then draw a smaller solid circle in white to create the centre hole
    } else {
      ePaper::circle(x, y, radius, false); // just a regular unfilled black circle
    }
    ePaper::filledRectangle(0, 170, 199, 199, true); // erase any portions of circles near bottom of display
    ePaper::text(12, 183, "ceptimus.co.uk", false);
    ePaper::displayFrame();
  }  
  ePaper::sleep();
}

void demo3() { // drawing a graph
  ePaper::clearBuffer(true);
  // y axis text
  ePaper::text(5,   5, "4.2", false);
  ePaper::text(5,  65, "3.6", false);
  ePaper::text(5, 125, "3.0", false);

  // x axis text
  ePaper::textSmall( 82, 137, "30", false);
  ePaper::textSmall(122, 137, "60", false);
  ePaper::textSmall(162, 137, "90", false);
  
  // y axis with markers
  ePaper::line(48, 0, 48, 131, false);
  for (int y = 11; y <= 131; y += 10) {
    ePaper::line(45, y, 51, y, false);  
  }

  // x axis with markers
  ePaper::line(48, 131, 199, 131, false);
  for (int x = 48; x < 200; x += 20) {
    ePaper::line(x, 128, x, 134, false);  
  }

  ePaper::text(84, 17, "V =", false);
  ePaper::text(60, 104, "T =", false);

  ePaper::text(12, 183, "ceptimus.co.uk", false);

  ePaper::wake();
  ePaper::fullUpdateMode(); // do a full update the first time
  ePaper::displayFrame(); // display everything drawn so far
  ePaper::partialUpdateMode();

  int prevX = 48;
  int prevY = 11; // previous point on graph (x, y) coordinates
  for (float t = 0.0; t < 7.6; t += 0.2) {
    float v = 4.2 - t * 0.142 - sin(t) * 0.125; // very rough mathematical model for the discharge curve of a Li-Ion cell volts against time (arbitrary time units)
    int x = 48 + t * 19;
    int y = 433 - v * 100; // convert from (t, v) coordinates (voltage/time) to (x, y) pixel coordinates
    ePaper::line(prevX, prevY, x, y, false); // draw a straight line linking the previous coordinate to the new one
    ePaper::line(prevX, prevY + 1, x, y + 1, false); // draw it again one pixel down to make a thicker line
    prevX = x;
    prevY = y;
    int minutes = (int)(t * 14); // convert from arbitrary time units to minutes
    
    char buffer[5]; // used to format the time and voltage for printing to the display
    
    buffer[0] = minutes / 60 + '0'; // hours
    buffer[1] = ':';
    buffer[2] = (minutes % 60) / 10 + '0'; // tens of minutes
    buffer[3] = minutes % 10 + '0'; // minutes 0-9
    buffer[4] = '\0'; // string terminator
    ePaper::text(108, 104, buffer, false);

    int centivolts = v * 100;
    buffer[0] = centivolts / 100 + '0'; // whole volts
    buffer[1] = '.';
    buffer[2] = (centivolts % 100) / 10 + '0'; // tenths volts
    buffer[3] = centivolts % 10 + '0'; // hundredths volts
    buffer[4] = '\0'; // string terminator
    ePaper::text(132, 17, buffer, false);
    
    ePaper::displayFrame();  
  }
  ePaper::sleep();
}

void setup() {
  // Serial.begin(115200);
  ePaper::start();

  for (;;) { // loop forever
    demo1(); // some text and some parobolic curves made by curve stitching in a box
    demo2(); // random circles
    demo3(); // drawing a graph
  }
}

void loop() {
}

